@@ -10,7 +10,7 @@
 
 /***********************************************************************
  Copyright (c) 1998 by Kevin Atkinson, (c) 1999, 2000 and 2001 by
- MySQL AB, and (c) 2004, 2005 by Educational Technology Resources, Inc.
+ MySQL AB, and (c) 2004-2007 by Educational Technology Resources, Inc.
  Others may also hold copyrights on code in this file.  See the CREDITS
  file in the top directory of the distribution for details.
 
@@ -35,25 +35,24 @@
 #ifndef MYSQLPP_CONNECTION_H
 #define MYSQLPP_CONNECTION_H
 
-#include "platform.h"
-
-#include "defs.h"
+#include "common.h"
 
 #include "lockable.h"
 #include "noexceptions.h"
 
-#include <mysql.h>
-
 #include <deque>
 #include <string>
 
 namespace mysqlpp {
 
-class Query;
+#if !defined(DOXYGEN_IGNORE)
+// Make Doxygen ignore this
+class MYSQLPP_EXPORT Query;
+#endif
 
 /// \brief Manages the connection to the MySQL database.
 
-class Connection : public OptionalExceptions, public Lockable
+class MYSQLPP_EXPORT Connection : public OptionalExceptions, public Lockable
 {
 public:
 	/// \brief Legal types of option arguments
@@ -101,6 +100,10 @@ public:
 		// Set reporting of data truncation errors
 		opt_report_data_truncation,
 
+		// Enable or disable automatic reconnection to the server if
+		// the connection is found to have been lost.
+		opt_reconnect,
+
 		// Number of options supported.  Never send this to
 		// set_option()!
 		opt_COUNT
@@ -109,7 +112,7 @@ public:
 	/// \brief Create object without connecting it to the MySQL server.
 	///
 	/// \param te if true, exceptions are thrown on errors
-	MYSQLPP_EXPORT Connection(bool te = true);
+	Connection(bool te = true);
 
 	/// \brief Create object and connect to database server in one step.
 	///
@@ -137,14 +140,26 @@ public:
 	///		running on, or 0 to use default name
 	///	\param client_flag special connection flags. See MySQL C API
 	///		documentation for \c mysql_real_connect() for details.
-	MYSQLPP_EXPORT Connection(const char* db, const char* host = "",
+	Connection(const char* db, const char* host = "",
 			const char* user = "", const char* passwd = "",
 			uint port = 0, my_bool compress = 0,
 			unsigned int connect_timeout = 60, cchar* socket_name = 0,
 			unsigned int client_flag = 0);
 
+	/// \brief Establish a new connection using the same parameters as
+	/// an existing C API connection.
+	///
+	/// \param other existing Connection object
+	Connection(const Connection& other);
+
+	/// \brief Establish a new connection using the same parameters as
+	/// an existing C API connection.
+	///
+	/// \param mysql existing MySQL C API connection object
+	bool connect(const MYSQL& mysql);
+
 	/// \brief Destroy connection object
-	MYSQLPP_EXPORT ~Connection();
+	~Connection();
 
 	/// \brief Connect to database after object is created.
 	///
@@ -154,7 +169,7 @@ public:
 	/// If you call this method on an object that is already connected
 	/// to a database server, the previous connection is dropped and a
 	/// new connection is established.
-	MYSQLPP_EXPORT bool connect(cchar* db = "", cchar* host = "",
+	bool connect(cchar* db = "", cchar* host = "",
 			cchar* user = "", cchar* passwd = "", uint port = 0,
 			my_bool compress = 0, unsigned int connect_timeout = 60,
 			cchar* socket_name = 0, unsigned int client_flag = 0);
@@ -170,7 +185,7 @@ public:
 	
 	/// \brief Calls MySQL C API function \c mysql_info() and returns
 	/// result as a C++ string.
-	MYSQLPP_EXPORT std::string info();
+	std::string info();
 
 	/// \brief return true if connection was established successfully
 	///
@@ -196,7 +211,7 @@ public:
 	/// \link mysqlpp::Query::execute() execute() \endlink
 	/// on that object, the query is sent to the server this object
 	/// is connected to.
-	MYSQLPP_EXPORT Query query();
+	Query query();
 
 	/// \brief Alias for success()
 	///
@@ -215,6 +230,10 @@ public:
 	/// \endcode
 	operator bool() { return success(); }
 
+	/// \brief Copy an existing Connection object's state into this
+	/// object.
+	Connection& operator=(const Connection& rhs);
+
 	/// \brief Return error message for last MySQL error associated with
 	/// this connection.
 	///
@@ -245,15 +264,16 @@ public:
 
 	/// \brief "Pings" the MySQL database
 	///
-	/// If server doesn't respond, this function tries to reconnect.
+	/// Wraps \c mysql_ping() in the C API.  As a result, this function
+	/// will try to reconnect to the server if the connection has been
+	/// dropped.
 	/// 
 	/// \retval 0 if server is responding, regardless of whether we had
 	/// to reconnect or not
-	/// \retval nonzero if server did not respond to ping and we could
-	/// not re-establish the connection
-	///
-	/// Simply wraps \c mysql_ping() in the C API.
-	int ping() { return mysql_ping(&mysql_); }
+	/// \retval nonzero if either we already know the connection is down
+	/// and cannot re-establish it, or if the server did not respond to
+	/// the ping and we could not re-establish the connection.
+	int ping();
 
 	/// \brief Kill a MySQL server thread
 	///
@@ -317,14 +337,14 @@ public:
 	/// \param db name of database to create
 	///
 	/// \return true if database was created successfully
-	MYSQLPP_EXPORT bool create_db(const std::string& db);
+	bool create_db(const std::string& db);
 
 	/// \brief Drop a database
 	///
 	/// \param db name of database to destroy
 	///
 	/// \return true if database was created successfully
-	MYSQLPP_EXPORT bool drop_db(const std::string& db);
+	bool drop_db(const std::string& db);
 
 	/// \brief Change to a different database
 	bool select_db(const std::string& db)
@@ -333,7 +353,7 @@ public:
 	}
 
 	/// \brief Change to a different database
-	MYSQLPP_EXPORT bool select_db(const char* db);
+	bool select_db(const char* db);
 
 	/// \brief Ask MySQL server to reload the grant tables
 	/// 
@@ -342,14 +362,14 @@ public:
 	/// Simply wraps \c mysql_reload() in the C API.  Since that
 	/// function is deprecated, this one is, too.  The MySQL++
 	/// replacement is execute("FLUSH PRIVILEGES").
-	MYSQLPP_EXPORT bool reload();
+	bool reload();
 	
 	/// \brief Ask MySQL server to shut down.
 	///
 	/// User must have the "shutdown" privilege.
 	///
 	/// Simply wraps \c mysql_shutdown() in the C API.
-	MYSQLPP_EXPORT bool shutdown();
+	bool shutdown();
 
 	/// \brief Return the connection options object
 	st_mysql_options get_options() const
@@ -379,7 +399,8 @@ public:
 	/// C API's option setting mechanisms, so it may become further
 	/// abstracted from these mechanisms.
 	///
-	/// \retval true if option was successfully set
+	/// \retval true if option was successfully set, or at least queued
+	/// for setting during connection establishment sequence
 	///
 	/// If exceptions are enabled, a false return means the C API
 	/// rejected the option, or the connection is not established and
@@ -389,16 +410,28 @@ public:
 	/// of range, or the option is not supported by the C API, most
 	/// because it isn't a high enough version. These latter cases will
 	/// cause BadOption exceptions otherwise.
-	MYSQLPP_EXPORT bool set_option(Option option);
+	bool set_option(Option option);
 
 	/// \brief Sets a connection option, with string argument
-	MYSQLPP_EXPORT bool set_option(Option option, const char* arg);
+	bool set_option(Option option, const char* arg);
 
 	/// \brief Sets a connection option, with integer argument
-	MYSQLPP_EXPORT bool set_option(Option option, unsigned int arg);
+	bool set_option(Option option, unsigned int arg);
 
 	/// \brief Sets a connection option, with Boolean argument
-	MYSQLPP_EXPORT bool set_option(Option option, bool arg);
+	bool set_option(Option option, bool arg);
+
+	/// \brief Same as set_option(), except that it won't override
+	/// a previously-set option.
+	bool set_option_default(Option option);
+
+	/// \brief Same as set_option(), except that it won't override
+	/// a previously-set option.
+	template <typename T>
+	bool set_option_default(Option option, T arg);
+
+	/// \brief Returns true if the given option has been set already
+	bool option_set(Option option);
 
 	/// \brief Enable SSL-encrypted connection.
 	///
@@ -412,7 +445,7 @@ public:
 	/// Must be called before connection is established.
 	///
 	/// Wraps \c mysql_ssl_set() in MySQL C API.
-	MYSQLPP_EXPORT void enable_ssl(const char* key = 0,
+	void enable_ssl(const char* key = 0,
 			const char* cert = 0, const char* ca = 0,
 			const char* capath = 0, const char* cipher = 0);
 
@@ -442,35 +475,22 @@ public:
 	std::ostream& api_version(std::ostream& os);
 
 protected:
+	/// \brief Types of option setting errors we can diagnose
+	enum OptionError {
+		opt_err_type,
+		opt_err_value,
+		opt_err_conn
+	};
+	
 	/// \brief Drop the connection to the database server
 	///
 	/// This method is protected because it should only be used within
 	/// the library.  Unless you use the default constructor, this
 	/// object should always be connected.
-	MYSQLPP_EXPORT void disconnect();
+	void disconnect();
 
-	/// \brief Returns true if the given option is to be set once
-	/// connection comes up.
-	///
-	/// \param option option to check for in queue
-	/// \param arg argument to match against
-	bool option_pending(Option option, bool arg) const;
-
-	/// \brief For each option in pending option queue, call
-	/// set_option()
-	///
-	/// Called within connect() method after connection is established.
-	/// Despools options in the order given to set_option().
-	void apply_pending_options();
-
-	/// \brief Generic wrapper for bad_option_*()
-	bool bad_option(Option option, OptionArgType type);
-
-	/// \brief Handles call of incorrect set_option() overload
-	bool bad_option_type(Option option);
-
-	/// \brief Handles bad option values sent to set_option()
-	bool bad_option_value(Option option);
+	/// \brief Error handling routine for set_option()
+	bool bad_option(Option option, OptionError error);
 
 	/// \brief Given option value, return its proper argument type
 	OptionArgType option_arg_type(Option option);
@@ -491,6 +511,11 @@ protected:
 	bool set_option_impl(enum_mysql_set_option msoption);
 #endif
 
+	/// \brief Establish a new connection as a copy of an existing one
+	///
+	/// \param other the connection to copy
+	void copy(const Connection& other);
+
 private:
 	friend class ResNSel;
 	friend class ResUse;
@@ -536,12 +561,14 @@ private:
 		{
 		}
 	};
+	typedef std::deque<OptionInfo> OptionList;
+	typedef OptionList::const_iterator OptionListIt;
 
 	MYSQL mysql_;
 	bool is_connected_;
 	bool connecting_;
 	bool success_;
-	std::deque<OptionInfo> pending_options_;
+	OptionList applied_options_;
 	static OptionArgType legal_opt_arg_types_[];
 };
 
