@@ -2,7 +2,7 @@
  query.cpp - Implements the Query class.
 
  Copyright (c) 1998 by Kevin Atkinson, (c) 1999, 2000 and 2001 by
- MySQL AB, and (c) 2004, 2005 by Educational Technology Resources, Inc.
+ MySQL AB, and (c) 2004-2007 by Educational Technology Resources, Inc.
  Others may also hold copyrights on code in this file.  See the CREDITS
  file in the top directory of the distribution for details.
 
@@ -26,13 +26,36 @@
 
 #include "query.h"
 
+#include "autoflag.h"
 #include "connection.h"
 
 namespace mysqlpp {
 
+Query::Query(Connection* c, bool te) :
+#if defined(_MSC_VER) && !defined(_STLP_VERSION) && !defined(_STLP_VERSION_STR)
+// prevents a double-init memory leak in native VC++ RTL (not STLport!)
+std::ostream(std::_Noinit), 
+#else
+std::ostream(0),
+#endif
+OptionalExceptions(te),
+Lockable(false),
+def(this),
+conn_(c),
+success_(false)
+{
+	init(&sbuffer_);
+	success_ = true;
+}
+
 Query::Query(const Query& q) :
-std::ostream(0),	
-OptionalExceptions(q.exceptions()),
+#if defined(_MSC_VER) && !defined(_STLP_VERSION) && !defined(_STLP_VERSION_STR)
+// ditto above
+std::ostream(std::_Noinit),
+#else
+std::ostream(0),
+#endif
+OptionalExceptions(q.throw_exceptions()),
 Lockable(q.locked()),
 def(q.def),
 conn_(q.conn_),
@@ -55,21 +78,24 @@ Query::operator=(const Query& rhs)
 }
 
 
-my_ulonglong Query::affected_rows() const
+my_ulonglong
+Query::affected_rows() const
 {
 	return conn_->affected_rows();
 }
 
 
-std::string Query::error()
+std::string
+Query::error()
 {
 	return conn_->error();
 }
 
 
-bool Query::exec(const std::string& str)
+bool
+Query::exec(const std::string& str)
 {
-	success_ = !mysql_real_query(&conn_->mysql_, str.c_str(),
+	success_ = !mysql_real_query(&conn_->mysql_, str.data(),
 			static_cast<unsigned long>(str.length()));
 	if (!success_ && throw_exceptions()) {
 		throw BadQuery(error());
@@ -80,10 +106,37 @@ bool Query::exec(const std::string& str)
 }
 
 
-ResNSel Query::execute(const char* str)
+ResNSel
+Query::execute(const SQLString& str)
+{
+	if ((parse_elems_.size() == 2) && !def.processing_) {
+		// We're a template query and we haven't gone through this path
+		// before, so take str to be a lone parameter for the query.
+		// We will come back through this function with a completed
+		// query, but the processing_ flag will be reset, allowing us to
+		// take the 'else' path, avoiding an infinite loop.
+		AutoFlag<> af(def.processing_);
+		return execute(SQLQueryParms() << str);
+	}
+	else {
+		// Take str to be the entire query string
+		return execute(str.data(), str.length());
+	}
+}
+
+
+ResNSel
+Query::execute(const char* str)
+{
+	return execute(SQLString(str));
+}
+
+
+ResNSel
+Query::execute(const char* str, size_t len)
 {
-	success_ = false;
 	if (lock()) {
+		success_ = false;
 		if (throw_exceptions()) {
 			throw LockFailed();
 		}
@@ -92,18 +145,17 @@ ResNSel Query::execute(const char* str)
 		}
 	}
 
-	success_ = !mysql_query(&conn_->mysql_, str);
+	success_ = !mysql_real_query(&conn_->mysql_, str, len);
+
 	unlock();
 	if (success_) {
 		return ResNSel(conn_);
 	}
+	else if (throw_exceptions()) {
+		throw BadQuery(error());
+	}
 	else {
-		if (throw_exceptions()) {
-			throw BadQuery(error());
-		}
-		else {
-			return ResNSel();
-		}
+		return ResNSel();
 	}
 }
 
@@ -111,34 +163,38 @@ ResNSel Query::execute(const char* str)
 #if !defined(DOXYGEN_IGNORE)
 // Doxygen will not generate documentation for this section.
 
-ResNSel Query::execute(SQLQueryParms& p)
+ResNSel
+Query::execute(SQLQueryParms& p)
 {
-	query_reset r = parse_elems_.size() ? DONT_RESET : RESET_QUERY;
-	return execute(str(p, r).c_str());
+	return execute(str(p, parse_elems_.size() ? DONT_RESET : RESET_QUERY));
 }
 
 #endif // !defined(DOXYGEN_IGNORE)
 
 
-std::string Query::info()
+std::string
+Query::info()
 {
 	return conn_->info();
 }
 
 
-my_ulonglong Query::insert_id()
+my_ulonglong
+Query::insert_id()
 {
 	return conn_->insert_id();
 }
 
 
-bool Query::lock()
+bool
+Query::lock()
 {
     return conn_->lock();
 }
 
 
-bool Query::more_results()
+bool 
+Query::more_results()
 {
 #if MYSQL_VERSION_ID > 41000		// only in MySQL v4.1 +
 	return mysql_more_results(&conn_->mysql_);
@@ -148,7 +204,8 @@ bool Query::more_results()
 }
 
 
-void Query::parse()
+void
+Query::parse()
 {
 	std::string str = "";
 	char num[4];
@@ -185,7 +242,7 @@ void Query::parse()
 				else {
 					num[1] = 0;
 				}
-				short int n = atoi(num);
+				signed char n = atoi(num);
 
 				// Look for option character following position value.
 				char option = ' ';
@@ -220,7 +277,7 @@ void Query::parse()
 				}
 
 				// Finished parsing parameter; save it.
-				parse_elems_.push_back(SQLParseElement(str, option, char(n)));
+				parse_elems_.push_back(SQLParseElement(str, option, n));
 				str = "";
 				name = "";
 			}
@@ -252,7 +309,7 @@ Query::pprepare(char option, SQLString& S, bool replace)
 
 	if (option == 'r' || (option == 'q' && S.is_string)) {
 		char *s = new char[S.size() * 2 + 1];
-		mysql_real_escape_string(&conn_->mysql_, s, S.c_str(),
+		mysql_real_escape_string(&conn_->mysql_, s, S.data(),
 				static_cast<unsigned long>(S.size()));
 		SQLString *ss = new SQLString("'");
 		*ss += s;
@@ -262,6 +319,7 @@ Query::pprepare(char option, SQLString& S, bool replace)
 		if (replace) {
 			S = *ss;
 			S.processed = true;
+			delete ss;
 			return &S;
 		}
 		else {
@@ -274,6 +332,7 @@ Query::pprepare(char option, SQLString& S, bool replace)
 		if (replace) {
 			S = *ss;
 			S.processed = true;
+			delete ss;
 			return &S;
 		}
 		else {
@@ -289,48 +348,54 @@ Query::pprepare(char option, SQLString& S, bool replace)
 }
 
 
-char* Query::preview_char()
+char*
+Query::preview_char()
 {
-	*this << std::ends;
-	size_t length = sbuffer_.str().size();
-	char* s = new char[length + 1];
-	strcpy(s, sbuffer_.str().c_str());
+	const std::string& str(sbuffer_.str());
+	char* s = new char[str.size() + 1];
+	memcpy(s, str.data(), str.size()); 
+	s[str.size()] = '\0';
 	return s;
 }
 
 
-void Query::proc(SQLQueryParms& p)
+void
+Query::proc(SQLQueryParms& p)
 {
 	sbuffer_.str("");
 
-	char num;
-	SQLString* ss;
-	SQLQueryParms* c;
-
 	for (std::vector<SQLParseElement>::iterator i = parse_elems_.begin();
 			i != parse_elems_.end(); ++i) {
-		dynamic_cast<std::ostream&>(*this) << i->before;
-		num = i->num;
-		if (num != -1) {
-			if (num < static_cast<int>(p.size()))
+		MYSQLPP_QUERY_THISPTR << i->before;
+		int num = i->num;
+		if (num >= 0) {
+			SQLQueryParms* c;
+			if (size_t(num) < p.size()) {
 				c = &p;
-			else if (num < static_cast<int>(def.size()))
+			}
+			else if (size_t(num) < def.size()) {
 				c = &def;
+			}
 			else {
 				*this << " ERROR";
 				throw BadParamCount(
 						"Not enough parameters to fill the template.");
 			}
-			ss = pprepare(i->option, (*c)[num], c->bound());
-			dynamic_cast<std::ostream&>(*this) << *ss;
-			if (ss != &(*c)[num]) {
+
+			SQLString& param = (*c)[num];
+			SQLString* ss = pprepare(i->option, param, c->bound());
+			MYSQLPP_QUERY_THISPTR << *ss;
+			if (ss != &param) {
+				// pprepare() returned a new string object instead of
+				// updating param in place, so we need to delete it.
 				delete ss;
 			}
 		}
 	}
 }
 
-void Query::reset()
+void
+Query::reset()
 {
 	seekp(0);
 	clear();
@@ -341,11 +406,37 @@ void Query::reset()
 }
 
 
-Result Query::store(const char* str)
+Result 
+Query::store(const SQLString& str)
 {
-	success_ = false;
+	if ((parse_elems_.size() == 2) && !def.processing_) {
+		// We're a template query and we haven't gone through this path
+		// before, so take str to be a lone parameter for the query.
+		// We will come back through this function with a completed
+		// query, but the processing_ flag will be reset, allowing us to
+		// take the 'else' path, avoiding an infinite loop.
+		AutoFlag<> af(def.processing_);
+		return store(SQLQueryParms() << str);
+	}
+	else {
+		// Take str to be the entire query string
+		return store(str.data(), str.length());
+	}
+}
 
+
+Result
+Query::store(const char* str)
+{
+	return store(SQLString(str));
+}
+
+
+Result
+Query::store(const char* str, size_t len)
+{
 	if (lock()) {
+		success_ = false;
 		if (throw_exceptions()) {
 			throw LockFailed();
 		}
@@ -354,22 +445,26 @@ Result Query::store(const char* str)
 		}
 	}
 
-	success_ = !mysql_query(&conn_->mysql_, str);
-	if (success_) {
+
+	if (success_ = !mysql_real_query(&conn_->mysql_, str, len)) {
 		MYSQL_RES* res = mysql_store_result(&conn_->mysql_);
 		if (res) {
 			unlock();
 			return Result(res, throw_exceptions());
 		}
+		else {
+			success_ = false;
+		}
 	}
 	unlock();
 
-	// One of the mysql_* calls failed, so decide how we should fail.
-	// Notice that we do not throw an exception if we just get a null
-	// result set, but no error.  This happens when using store() on a
-	// query that may not return results.  Obviously it's better to use
-	// exec*() in this situation, but it's not outright illegal, and
-	// sometimes you have to do it.
+	// One of the MySQL API calls failed, but it's not an error if we
+	// just get an empty result set.  It happens when store()ing a query
+	// that doesn't always return results.  While it's better to use 
+	// exec*() in that situation, it's legal to call store() instead,
+	// and sometimes you have no choice.  For example, if the SQL comes
+	// from outside the program so you can't predict whether there will
+	// be results.
 	if (conn_->errnum() && throw_exceptions()) {
 		throw BadQuery(error());
 	}
@@ -382,16 +477,17 @@ Result Query::store(const char* str)
 #if !defined(DOXYGEN_IGNORE)
 // Doxygen will not generate documentation for this section.
 
-Result Query::store(SQLQueryParms& p)
+Result
+Query::store(SQLQueryParms& p)
 {
-	query_reset r = parse_elems_.size() ? DONT_RESET : RESET_QUERY;
-	return store(str(p, r).c_str());
+	return store(str(p, parse_elems_.size() ? DONT_RESET : RESET_QUERY));
 }
 
 #endif // !defined(DOXYGEN_IGNORE)
 
 
-Result Query::store_next()
+Result
+Query::store_next()
 {
 #if MYSQL_VERSION_ID > 41000		// only in MySQL v4.1 +
 	if (lock()) {
@@ -445,19 +541,19 @@ Result Query::store_next()
 }
 
 
-std::string Query::str(SQLQueryParms& p)
+std::string
+Query::str(SQLQueryParms& p)
 {
 	if (!parse_elems_.empty()) {
 		proc(p);
 	}
 
-	*this << std::ends;
-
 	return sbuffer_.str();
 }
 
 
-std::string Query::str(SQLQueryParms& p, query_reset r)
+std::string
+Query::str(SQLQueryParms& p, query_reset r)
 {
 	std::string tmp = str(p);
 	if (r == RESET_QUERY) {
@@ -467,22 +563,51 @@ std::string Query::str(SQLQueryParms& p, query_reset r)
 }
 
 
-bool Query::success()
+bool
+Query::success()
 {
 	return success_ && conn_->success();
 }
 
 
-void Query::unlock()
+void
+Query::unlock()
 {
 	conn_->unlock();
 }
 
 
-ResUse Query::use(const char* str)
+ResUse
+Query::use(const SQLString& str)
+{
+	if ((parse_elems_.size() == 2) && !def.processing_) {
+		// We're a template query and we haven't gone through this path
+		// before, so take str to be a lone parameter for the query.
+		// We will come back through this function with a completed
+		// query, but the processing_ flag will be reset, allowing us to
+		// take the 'else' path, avoiding an infinite loop.
+		AutoFlag<> af(def.processing_);
+		return use(SQLQueryParms() << str);
+	}
+	else {
+		// Take str to be the entire query string
+		return use(str.data(), str.length());
+	}
+}
+
+
+ResUse
+Query::use(const char* str)
+{
+	return use(SQLString(str));
+}
+
+
+ResUse
+Query::use(const char* str, size_t len)
 {
-	success_ = false;
 	if (lock()) {
+		success_ = false;
 		if (throw_exceptions()) {
 			throw LockFailed();
 		}
@@ -491,8 +616,7 @@ ResUse Query::use(const char* str)
 		}
 	}
 
-	success_ = !mysql_query(&conn_->mysql_, str);
-	if (success_) {
+	if (success_ = !mysql_real_query(&conn_->mysql_, str, len)) {
 		MYSQL_RES* res = mysql_use_result(&conn_->mysql_);
 		if (res) {
 			unlock();
@@ -501,8 +625,14 @@ ResUse Query::use(const char* str)
 	}
 	unlock();
 
-	// One of the mysql_* calls failed, so decide how we should fail.
-	if (throw_exceptions()) {
+	// One of the MySQL API calls failed, but it's not an error if we
+	// just get an empty result set.  It happens when use()ing a query
+	// that doesn't always return results.  While it's better to use 
+	// exec*() in that situation, it's legal to call use() instead, and
+	// sometimes you have no choice.  For example, if the SQL comes
+	// from outside the program so you can't predict whether there will
+	// be results.
+	if (conn_->errnum() && throw_exceptions()) {
 		throw BadQuery(error());
 	}
 	else {
@@ -514,10 +644,10 @@ ResUse Query::use(const char* str)
 #if !defined(DOXYGEN_IGNORE)
 // Doxygen will not generate documentation for this section.
 
-ResUse Query::use(SQLQueryParms& p)
+ResUse
+Query::use(SQLQueryParms& p)
 {
-	query_reset r = parse_elems_.size() ? DONT_RESET : RESET_QUERY;
-	return use(str(p, r).c_str());
+	return use(str(p, parse_elems_.size() ? DONT_RESET : RESET_QUERY));
 }
 
 #endif // !defined(DOXYGEN_IGNORE)
