@@ -3,7 +3,7 @@
 
 /***********************************************************************
  Copyright (c) 1998 by Kevin Atkinson, (c) 1999, 2000 and 2001 by
- MySQL AB, and (c) 2004, 2005 by Educational Technology Resources, Inc.
+ MySQL AB, and (c) 2004-2006 by Educational Technology Resources, Inc.
  Others may also hold copyrights on code in this file.  See the CREDITS
  file in the top directory of the distribution for details.
 
@@ -28,18 +28,18 @@
 #ifndef MYSQLPP_QUERY_H
 #define MYSQLPP_QUERY_H
 
-#include "defs.h"
+#include "common.h"
 
 #include "lockable.h"
 #include "noexceptions.h"
 #include "qparms.h"
+#include "querydef.h"
 #include "result.h"
 #include "row.h"
 #include "sql_string.h"
 
-#include <mysql.h>
-
 #include <deque>
+#include <iomanip>
 #include <list>
 #include <map>
 #include <set>
@@ -54,71 +54,25 @@
 #  endif
 #endif
 
-
-/// \brief Used to define many similar functions in class Query.
-#define mysql_query_define0(RETURN, FUNC)\
-  RETURN FUNC (ss a)\
-    {return FUNC (parms() << a);}\
-  RETURN FUNC (ss a, ss b)\
-    {return FUNC (parms() << a << b);}\
-  RETURN FUNC (ss a, ss b, ss c)\
-    {return FUNC (parms() << a << b << c);}\
-  RETURN FUNC (ss a, ss b, ss c, ss d)\
-    {return FUNC (parms() << a << b << c << d);}\
-  RETURN FUNC (ss a, ss b, ss c, ss d, ss e)\
-    {return FUNC (parms() << a << b << c << d << e);} \
-  RETURN FUNC (ss a, ss b, ss c, ss d, ss e, ss f)\
-    {return FUNC (parms() << a << b << c << d << e << f);}\
-  RETURN FUNC (ss a, ss b, ss c, ss d, ss e, ss f, ss g)\
-    {return FUNC (parms() << a << b << c << d << e << f << g);}\
-  RETURN FUNC (ss a, ss b, ss c, ss d, ss e, ss f, ss g, ss h)\
-    {return FUNC (parms() << a << b << c << d << e << f << g << h);}\
-  RETURN FUNC (ss a, ss b, ss c, ss d, ss e, ss f, ss g, ss h, ss i)\
-    {return FUNC (parms() << a << b << c << d << e << f << g << h << i);}\
-  RETURN FUNC (ss a,ss b,ss c,ss d,ss e,ss f,ss g,ss h,ss i,ss j)\
-    {return FUNC (parms() <<a <<b <<c <<d <<e <<f <<g <<h <<i <<j);}\
-  RETURN FUNC (ss a,ss b,ss c,ss d,ss e,ss f,ss g,ss h,ss i,ss j,ss k)\
-    {return FUNC (parms() <<a <<b <<c <<d <<e <<f <<g <<h <<i <<j <<k);}\
-  RETURN FUNC (ss a,ss b,ss c,ss d,ss e,ss f,ss g,ss h,ss i,ss j,ss k,ss l)\
-    {return FUNC (parms() <<a <<b <<c <<d <<e <<f <<g <<h <<i <<j <<k <<l);}\
-
-/// \brief Used to define many similar member functions in class Query.
-#define mysql_query_define1(RETURN, FUNC) \
-  MYSQLPP_EXPORT RETURN FUNC (parms &p);\
-  mysql_query_define0(RETURN,FUNC) \
-
-/// \brief Used to define many similar member functions in class Query.
-#define mysql_query_define2(FUNC) \
-  template <class T1> void FUNC (T1 &con, const char* str); \
-  template <class T1> void FUNC (T1 &con, parms &p, query_reset r = RESET_QUERY);\
-  template <class T1> void FUNC (T1 &con, ss a)\
-	{FUNC (con, parms() << a);}\
-  template <class T1> void FUNC (T1 &con, ss a, ss b)\
-	{FUNC (con, parms() << a << b);}\
-  template <class T1> void FUNC (T1 &con, ss a, ss b, ss c)\
-	{FUNC (con, parms() << a << b << c);}\
-  template <class T1> void FUNC (T1 &con, ss a, ss b, ss c, ss d)\
-	{FUNC (con, parms() << a << b << c << d);}\
-  template <class T1> void FUNC (T1 &con, ss a, ss b, ss c, ss d, ss e)\
-	{FUNC (con, parms() << a << b << c << d << e);} \
-  template <class T1> void FUNC (T1 &con, ss a, ss b, ss c, ss d, ss e, ss f)\
-	{FUNC (con, parms() << a << b << c << d << e << f);}\
-  template <class T1> void FUNC (T1 &con,ss a,ss b,ss c,ss d,ss e,ss f,ss g)\
-	{FUNC (con, parms() << a << b << c << d << e << f << g);}\
-  template <class T1> void FUNC (T1 &con,ss a,ss b,ss c,ss d,ss e,ss f,ss g,ss h)\
-	{FUNC (con, parms() << a << b << c << d << e << f << g << h);}\
-  template <class T1> void FUNC (T1 &con, ss a, ss b, ss c, ss d, ss e, ss f, ss g, ss h, ss i)\
-	{FUNC (con, parms() << a << b << c << d << e << f << g << h << i);}\
-  template <class T1> void FUNC (T1 &con, ss a,ss b,ss c,ss d,ss e,ss f,ss g,ss h,ss i,ss j)\
-	{FUNC (con, parms() <<a <<b <<c <<d <<e <<f <<g <<h <<i <<j);}\
-  template <class T1> void FUNC (T1 &con, ss a,ss b,ss c,ss d,ss e,ss f,ss g,ss h,ss i,ss j,ss k)\
-	{FUNC (con, parms() <<a <<b <<c <<d <<e <<f <<g <<h <<i <<j <<k);}\
-  template <class T1> void FUNC (T1 &con, ss a,ss b,ss c,ss d,ss e,ss f,ss g,ss h,ss i,ss j,ss k,ss l)\
-	{FUNC (con, parms() <<a <<b <<c <<d <<e <<f <<g <<h <<i <<j <<k <<l);}\
+/// \def MYSQLPP_QUERY_THISPTR
+/// \brief Helper macro used inside MySQL++ to work around a VC++ 2003 bug
+///
+/// This macro returns '*this', either directly or upcast to Query's
+/// base class to work around an error in the overloaded operator
+/// lookup logic in VC++ 2003.  For an explanation of the problem, see:
+/// http://groups.google.com/group/microsoft.public.vc.stl/browse_thread/thread/9a68d84644e64f15
+#if defined(_MSC_VER) && (_MSC_VER < 1400)
+#	define MYSQLPP_QUERY_THISPTR dynamic_cast<std::ostream&>(*this)
+#else
+#	define MYSQLPP_QUERY_THISPTR *this
+#endif
 
 namespace mysqlpp {
 
-class Connection;
+#if !defined(DOXYGEN_IGNORE)
+// Make Doxygen ignore this
+class MYSQLPP_EXPORT Connection;
+#endif
 
 /// \brief Used for indicating whether a query object should auto-reset
 enum query_reset { DONT_RESET, RESET_QUERY };
@@ -165,37 +119,24 @@ enum query_reset { DONT_RESET, RESET_QUERY };
 /// You call the parse() method to tell the Query object that the query
 /// string contains placeholders. Once that's done, you can call any of
 /// the many overloaded methods that take a number of SQLStrings (up to
-/// 12 at the moment) or any type that can be converted to SQLString,
-/// and those parameters will be inserted into the placeholders. When
-/// you call one of the parameterless functions the execute the query,
-/// the final query string is assembled and sent to the server.
+/// 25 by default) or any type that can be converted to SQLString, and
+/// those parameters will be inserted into the placeholders. When you
+/// call one of the parameterless functions the execute the query, the
+/// final query string is assembled and sent to the server.
 ///
 /// See the user manual for more details about these options.
 
-class Query : public std::ostream,
+class MYSQLPP_EXPORT Query : public std::ostream,
 		public OptionalExceptions, public Lockable
 {
 public:
-	typedef const SQLString& ss;	///< to keep parameter lists short
-	typedef SQLQueryParms parms;	///< shortens def'ns of above macros
-
 	/// \brief Create a new query object attached to a connection.
 	///
 	/// This is the constructor used by mysqlpp::Connection::query().
 	///
 	/// \param c connection the finished query should be sent out on
 	/// \param te if true, throw exceptions on errors
-	Query(Connection* c, bool te = true) :
-	std::ostream(0),
-	OptionalExceptions(te),
-	Lockable(false),
-	def(this),
-	conn_(c),
-	success_(false)
-	{
-		init(&sbuffer_);
-		success_ = true;
-	}
+	Query(Connection* c, bool te = true);
 
 	/// \brief Create a new query object as a copy of another.
 	///
@@ -204,27 +145,27 @@ public:
 	/// to an empty Query object.  In particular, the stream buffer and
 	/// template query stuff will be empty in the copy, regardless of
 	/// what values they have in the original.
-	MYSQLPP_EXPORT Query(const Query& q);
+	Query(const Query& q);
 
 	/// \brief Assign another query's state to this object
 	///
 	/// The same caveats apply to this operator as apply to the copy
 	/// ctor.
-	MYSQLPP_EXPORT Query& operator=(const Query& rhs);
+	Query& operator=(const Query& rhs);
 
 	/// \brief Get the last error message that was set.
 	///
 	/// This class has an internal error message string, but if it
 	/// isn't set, we return the last error message that happened
 	/// on the connection we're bound to instead.
-	MYSQLPP_EXPORT std::string error();
+	std::string error();
 
 	/// \brief Returns true if the last operation succeeded
 	///
 	/// Returns true if the last query succeeded, and the associated
 	/// Connection object's success() method also returns true.  If
 	/// either object is unhappy, this method returns false.
-	MYSQLPP_EXPORT bool success();
+	bool success();
 
 	/// \brief Treat the contents of the query string as a template
 	/// query.
@@ -233,43 +174,50 @@ public:
 	/// other members that accept template query parameters.  See the
 	/// "Template Queries" chapter in the user manual for more
 	/// information.
-	MYSQLPP_EXPORT void parse();
+	void parse();
 
 	/// \brief Reset the query object so that it can be reused.
 	///
 	/// This erases the query string and the contents of the parameterized
 	/// query element list.
-	MYSQLPP_EXPORT void reset();
+	void reset();
 
 	/// \brief Return the query string currently in the buffer.
 	std::string preview() { return str(def); }
 
+	/// \brief Return the query string currently in the buffer with
+	/// template query parameter substitution.
+	///
+	/// \param arg0 the value to substitute for the first template query
+	/// parameter
+	std::string preview(const SQLString& arg0)
+			{ return preview(SQLQueryParms() << arg0); }
+
 	/// \brief Return the query string currently in the buffer.
-	std::string preview(SQLQueryParms& p)
-	{
-		return str(p);
-	}
+	std::string preview(SQLQueryParms& p) { return str(p); }
 
 	/// \brief Get built query as a null-terminated C++ string
-	std::string str()
-	{
-		return str(def);
-	}
+	std::string str() { return str(def); }
+
+	/// \brief Get built query as a null-terminated C++ string with
+	/// template query parameter substitution.
+	///
+	/// \param arg0 the value to substitute for the first template query
+	/// parameter
+	std::string str(const SQLString& arg0)
+			{ return preview(SQLQueryParms() << arg0); }
 
 	/// \brief Get built query as a null-terminated C++ string
 	///
 	/// \param r if equal to \c RESET_QUERY, query object is cleared
 	/// after this call
-	std::string str(query_reset r)
-	{
-		return str(def, r);
-	}
+	std::string str(query_reset r) { return str(def, r); }
 
 	/// \brief Get built query as a null-terminated C++ string
 	///
 	/// \param p template query parameters to use, overriding the ones
 	/// this object holds, if any
-	MYSQLPP_EXPORT std::string str(SQLQueryParms& p);
+	std::string str(SQLQueryParms& p);
 
 	/// \brief Get built query as a null-terminated C++ string
 	///
@@ -277,7 +225,7 @@ public:
 	/// this object holds, if any
 	/// \param r if equal to \c RESET_QUERY, query object is cleared
 	/// after this call
-	MYSQLPP_EXPORT std::string str(SQLQueryParms& p, query_reset r);
+	std::string str(SQLQueryParms& p, query_reset r);
 
 	/// \brief Execute a query
 	///
@@ -290,7 +238,7 @@ public:
 	/// \return true if query was executed successfully
 	///
 	/// \sa execute(), store(), storein(), and use()
-	MYSQLPP_EXPORT bool exec(const std::string& str);
+	bool exec(const std::string& str);
 
 	/// \brief Execute built-up query
 	///
@@ -310,10 +258,24 @@ public:
 	/// \sa exec(), store(), storein(), and use()
 	ResNSel execute() { return execute(def); }
 
-	/// \brief Execute query in a C++ string
+	/// \brief Execute query in a C++ string, or substitute string into
+	/// a template query and execute it.
+	///
+	/// \param str If the object represents a compiled template query,
+	/// substitutes this string in for the first parameter.  Otherwise,
+	/// takes the string as a complete SQL query and executes it.
+	ResNSel execute(const SQLString& str);
+
+	/// \brief Execute query in a C string
 	///
 	/// Executes the query immediately, and returns the results.
-	MYSQLPP_EXPORT ResNSel execute(const char* str);
+	ResNSel execute(const char* str);
+
+	/// \brief Execute query in a known-length string of characters.
+	/// This can include null characters.
+	///
+	/// Executes the query immediately, and returns the results.
+	ResNSel execute(const char* str, size_t len);
 
 	/// \brief Execute a query that can return a result set
 	/// 
@@ -346,7 +308,21 @@ public:
 	/// Executes the query immediately, and returns an object that
 	/// lets you walk through the result set one row at a time, in
 	/// sequence.  This is more memory-efficient than store().
-	MYSQLPP_EXPORT ResUse use(const char* str);
+	ResUse use(const SQLString& str);
+
+	/// \brief Execute query in a C string
+	///
+	/// Executes the query immediately, and returns an object that
+	/// lets you walk through the result set one row at a time, in
+	/// sequence.  This is more memory-efficient than store().
+	ResUse use(const char* str);
+
+	/// \brief Execute query in a known-length C string
+	///
+	/// Executes the query immediately, and returns an object that
+	/// lets you walk through the result set one row at a time, in
+	/// sequence.  This is more memory-efficient than store().
+	ResUse use(const char* str, size_t len);
 
 	/// \brief Execute a query that can return a result set
 	///
@@ -376,7 +352,181 @@ public:
 	/// Executes the query immediately, and returns an object that
 	/// contains the entire result set.  This is less memory-efficient
 	/// than use(), but it lets you have random access to the results.
-	MYSQLPP_EXPORT Result store(const char* str);
+	Result store(const SQLString& str);
+
+	/// \brief Execute query in a C string
+	///
+	/// Executes the query immediately, and returns an object that
+	/// contains the entire result set.  This is less memory-efficient
+	/// than use(), but it lets you have random access to the results.
+	Result store(const char* str);
+
+	/// \brief Execute query in a known-length C string
+	///
+	/// Executes the query immediately, and returns an object that
+	/// contains the entire result set.  This is less memory-efficient
+	/// than use(), but it lets you have random access to the results.
+	Result store(const char* str, size_t len);
+
+	/// \brief Execute a query, and call a functor for each returned row
+	///
+	/// This method wraps a use() query, calling the given functor for
+	/// every returned row.  It is analogous to STL's for_each()
+	/// algorithm, but instead of iterating over some range within a
+	/// container, it iterates over a result set produced by a query.
+	///
+	/// \param query the query string
+	/// \param fn the functor called for each row
+	/// \return a copy of the passed functor
+	template <typename Function>
+	Function for_each(const SQLString& query, Function fn)
+	{	
+		mysqlpp::ResUse res = use(query);
+		if (res) {
+			mysqlpp::NoExceptions ne(res);
+			while (mysqlpp::Row row = res.fetch_row()) {
+				fn(row);
+			}
+		}
+
+		return fn;
+	}
+
+	/// \brief Execute the query, and call a functor for each returned row
+	///
+	/// Just like for_each(const SQLString&, Function), but it uses
+	/// the query string held by the Query object already
+	///
+	/// \param fn the functor called for each row
+	/// \return a copy of the passed functor
+	template <typename Function>
+	Function for_each(Function fn)
+	{	
+		mysqlpp::ResUse res = use();
+		if (res) {
+			mysqlpp::NoExceptions ne(res);
+			while (mysqlpp::Row row = res.fetch_row()) {
+				fn(row);
+			}
+		}
+
+		return fn;
+	}
+
+	/// \brief Run a functor for every row in a table
+	///
+	/// Just like for_each(Function), except that it builds a
+	/// "select * from TABLE" query using the SQL table name from
+	/// the SSQLS instance you pass.
+	///
+	/// \param ssqls the SSQLS instance to get a table name from
+	/// \param fn the functor called for each row
+	///
+	/// \return a copy of the passed functor
+	template <class SSQLS, typename Function>
+	Function for_each(const SSQLS& ssqls, Function fn)
+	{	
+		SQLString query("select * from ");
+		query += ssqls._table;
+		mysqlpp::ResUse res = use(query);
+		if (res) {
+			mysqlpp::NoExceptions ne(res);
+			while (mysqlpp::Row row = res.fetch_row()) {
+				fn(row);
+			}
+		}
+
+		return fn;
+	}
+
+	/// \brief Execute a query, conditionally storing each row in a
+	/// container
+	///
+	/// This method wraps a use() query, calling the given functor for
+	/// every returned row, and storing the results in the given
+	/// sequence container if the functor returns true.
+	///
+	/// This is analogous to the STL copy_if() algorithm, except that
+	/// the source rows come from a database query instead of another
+	/// container.  (copy_if() isn't a standard STL algorithm, but only
+	/// due to an oversight by the standardization committee.)  This
+	/// fact may help you to remember the order of the parameters: the
+	/// container is the destination, the query is the source, and the
+	/// functor is the predicate; it's just like an STL algorithm.
+	///
+	/// \param seq the destination container; needs a push_back() method
+	/// \param query the query string
+	/// \param fn the functor called for each row
+	/// \return a copy of the passed functor
+	template <class Sequence, typename Function>
+	Function store_if(Sequence& seq, const SQLString& query, Function fn)
+	{	
+		mysqlpp::ResUse res = use(query);
+		if (res) {
+			mysqlpp::NoExceptions ne(res);
+			while (mysqlpp::Row row = res.fetch_row()) {
+				if (fn(row)) {
+					seq.push_back(row);
+				}
+			}
+		}
+
+		return fn;
+	}
+
+	/// \brief Pulls every row in a table, conditionally storing each
+	/// one in a container
+	///
+	/// Just like store_if(Sequence&, const SQLString&, Function), but
+	/// it uses the SSQLS instance to construct a "select * from TABLE"
+	/// query, using the table name field in the SSQLS.
+	///
+	/// \param seq the destination container; needs a push_back() method
+	/// \param ssqls the SSQLS instance to get a table name from
+	/// \param fn the functor called for each row
+	/// \return a copy of the passed functor
+	template <class Sequence, class SSQLS, typename Function>
+	Function store_if(Sequence& seq, const SSQLS& ssqls, Function fn)
+	{	
+		SQLString query("select * from ");
+		query += ssqls._table;
+		mysqlpp::ResUse res = use(query);
+		if (res) {
+			mysqlpp::NoExceptions ne(res);
+			while (mysqlpp::Row row = res.fetch_row()) {
+				if (fn(row)) {
+					seq.push_back(row);
+				}
+			}
+		}
+
+		return fn;
+	}
+
+	/// \brief Execute the query, conditionally storing each row in a
+	/// container
+	///
+	/// Just like store_if(Sequence&, const SQLString&, Function), but
+	/// it uses the query string held by the Query object already
+	///
+	/// \param seq the destination container; needs a push_back() method
+	/// \param fn the functor called for each row
+	/// \return a copy of the passed functor
+	template <class Sequence, typename Function>
+	Function store_if(Sequence& seq, Function fn)
+	{	
+		mysqlpp::ResUse res = use();
+		if (res) {
+			mysqlpp::NoExceptions ne(res);
+			while (mysqlpp::Row row = res.fetch_row()) {
+				if (fn(row)) {
+					seq.push_back(row);
+				}
+			}
+		}
+
+		return fn;
+	}
 
 	/// \brief Return next result set, when processing a multi-query
 	///
@@ -404,7 +554,7 @@ public:
 	/// libraries.
 	///
 	/// \return Result object containing the next result set.
-	MYSQLPP_EXPORT Result store_next();
+	Result store_next();
 
 	/// \brief Return whether more results are waiting for a multi-query
 	/// or stored procedure response.
@@ -417,7 +567,7 @@ public:
 	/// always returns false when built against older API libraries.
 	///
 	/// \return true if another result set exists
-	MYSQLPP_EXPORT bool more_results();
+	bool more_results();
 
 	/// \brief Execute a query, storing the result set in an STL
 	/// sequence container.
@@ -439,7 +589,7 @@ public:
 	template <class Sequence>
 	void storein_sequence(Sequence& con, query_reset r = RESET_QUERY)
 	{
-		storein_sequence_(con, def, r);
+		storein_sequence(con, def, r);
 	}
 
 	/// \brief Execute a query, storing the result set in an STL
@@ -565,9 +715,9 @@ public:
 		// Cast required for VC++ 2003 due to error in overloaded operator
 		// lookup logic.  For an explanation of the problem, see:
 		// http://groups-beta.google.com/group/microsoft.public.vc.stl/browse_thread/thread/9a68d84644e64f15
-		dynamic_cast<std::ostream&>(*this) << "UPDATE " << o.table() <<
-				" SET " << n.equal_list() << " WHERE " <<
-				o.equal_list(" AND ", sql_use_compare);
+		MYSQLPP_QUERY_THISPTR << std::setprecision(16) <<
+				"UPDATE " << o.table() << " SET " << n.equal_list() <<
+				" WHERE " << o.equal_list(" AND ", sql_use_compare);
 		return *this;
 	}
 
@@ -584,10 +734,10 @@ public:
 	{
 		reset();
 
-		// See above comment for cast rationale
-		dynamic_cast<std::ostream&>(*this) << "INSERT INTO " <<
-				v.table() << " (" << v.field_list() << ") VALUES (" <<
-				v.value_list() << ")";
+		MYSQLPP_QUERY_THISPTR << std::setprecision(16) <<
+				"INSERT INTO " << v.table() << " (" <<
+				v.field_list() << ") VALUES (" <<
+				v.value_list() << ')';
 		return *this;
 	}
 
@@ -612,15 +762,14 @@ public:
 			return *this;	// empty set!
 		}
 		
-		// See above comment for cast rationale
-		dynamic_cast<std::ostream&>(*this) << "INSERT INTO " <<
-				first->table() << " (" << first->field_list() <<
-				") VALUES (" << first->value_list() << ')';
+		MYSQLPP_QUERY_THISPTR << std::setprecision(16) <<
+				"INSERT INTO " << first->table() << " (" <<
+				first->field_list() << ") VALUES (" <<
+				first->value_list() << ')';
 
 		Iter it = first + 1;
 		while (it != last) {
-			dynamic_cast<std::ostream&>(*this) << ",(" <<
-					it->value_list() << ')';
+			MYSQLPP_QUERY_THISPTR << ",(" << it->value_list() << ')';
 			++it;
 		}
 
@@ -641,10 +790,9 @@ public:
 	{
 		reset();
 
-		// See above comment for cast rationale
-		dynamic_cast<std::ostream&>(*this) << "REPLACE INTO " <<
-				v.table() << " (" << v.field_list() << ") VALUES (" <<
-				v.value_list() << ")";
+		MYSQLPP_QUERY_THISPTR << std::setprecision(16) <<
+				"REPLACE INTO " << v.table() << " (" <<
+				v.field_list() << ") VALUES (" << v.value_list() << ')';
 		return *this;
 	}
 
@@ -660,7 +808,7 @@ public:
 	// from Doxygen, which gets confused by macro instantiations that look
 	// like method declarations.
 	mysql_query_define0(std::string, preview)
-	mysql_query_define0(std::string, str);
+	mysql_query_define0(std::string, str)
 	mysql_query_define1(ResNSel, execute)
 	mysql_query_define1(Result, store)
 	mysql_query_define1(ResUse, use)
